package cc.smtweb.framework.core.db.vo;

import cc.smtweb.framework.core.annotation.SwTable;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.def.FieldType;
import cc.smtweb.framework.core.util.JsonUtil;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@Data
@SwTable(value="ASP_MODEL_TABLE")
public class ModelTable implements Serializable {
    private long id;
    //所属数据库id及名称
    private long databaseId;
    //项目id
    private long prjoectId;
    //目录id
    private long catalogId;
    //库表名
    private String name;
    //中文名
    private String title;
    //缩写
    private String abbr;
    //继承的表
    private String tableExtends;
    //类型 0-普通表，1 树型表 2 编码表 9-虚拟抽象表 11 视图'
    private int type;
    //表定义
    private String tableContent;
    //是否需要缓存
    private boolean needCache;
    //创建者
    private Long createUid;
    //创建时间
    private Long createAt;
    //最后更新人
    private Long updateUid;
    //更新时间
    private Long updateAt;

    /*冗余*/
    private String idField;
    private String dbName;

    private List<ModelField> fields = new ArrayList<>();
    private List<ModelIndex> indexes = new ArrayList<>();
    private List<ModelCache> caches = new ArrayList<>();

    public ModelField findField(String fieldName) {
        if (fieldName != null && fields != null) {
            for (ModelField modelField : fields) {
                if (fieldName.equals(modelField.getName())) {
                    return modelField;
                }
            }
        }

        return null;
    }

    public String findFieldTitle(String fieldName) {
        ModelField field = findField(fieldName);
        return field != null ? field.getTitle() : fieldName;
    }

    public String fullName() {
        return dbName + '.' + name;
    }

    public void addIndex(ModelIndex modelIndex) {
        indexes.add(modelIndex);
    }

    public ModelIndex findIndexByName(String indexName) {
        for (ModelIndex modelIndex : indexes) {
            if (indexName.equals(modelIndex.getName())) {
                return modelIndex;
            }
        }

        return null;
    }

    public ModelIndex findPrimaryIndex() {
        for (ModelIndex modelIndex : indexes) {
            if (ModelIndex.TYPE_PRIMARY.equalsIgnoreCase(modelIndex.getType())) {
                return modelIndex;
            }
        }

        return null;
    }

    public ModelField findFieldByName(String name) {
        if (name != null) {
            for (ModelField value : fields) {
                if (name.equalsIgnoreCase(value.getName())) {
                    return value;
                }
            }
        }

        return null;
    }

    public ModelField findFieldByType(FieldType type) {
        if (type != null) {
            for (ModelField value : fields) {
                if (type.name().equalsIgnoreCase(value.getFieldType())) {
                    return value;
                }
            }
        }

        return null;
    }

    public ModelField findIdField() {
        ModelIndex index = findPrimaryIndex();
        if (index != null) {
            return findField(index.getFields());
        }

        return null;
    }

    public List<ModelLinkName> findLinkeNames() {
        List<ModelLinkName> list = new ArrayList<>();
        for (ModelField field: fields) {
            if (StringUtils.isEmpty(field.getLink())) {
                continue;
            }
            ModelTable linkTable = ModelTableCache.getInstance().get(field.getLink());
            if (linkTable == null) {
                continue;
            }
            ModelField linkNameField = linkTable.findFieldByType(FieldType.NAME);
            if (linkNameField != null) {
                list.add(new ModelLinkName(field.getName(), linkTable, linkNameField.getName()));
            }
        }
        return list;
    }

    public void setTableContent(String tableContent) {
        this.tableContent = tableContent;
        //读取表定义信息
        ModelTable bean = JsonUtil.parse(tableContent, ModelTable.class);
        if (bean == null) {
            return;
        }
        this.fields = bean.fields;
        this.indexes = bean.indexes;
        this.caches = bean.caches;

        ModelIndex i = findPrimaryIndex();
        if (i != null) {
            this.idField = i.getFields();
        }
    }
}
