package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.R;
import cc.smtweb.framework.core.SwMap;
import cc.smtweb.framework.core.cache.SessionCache;
import cc.smtweb.framework.core.cache.SessionCacheFactory;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.mvc.service.list.FooterField;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.NumberUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.ResultSetExtractor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Akmm at 2022/3/2 19:44
 * 列表服务
 */
@Slf4j
public abstract class AbstractListHandler implements IHandler {
    protected static String KEY_SQLPARA = "query-sqlPara";
    protected static String KEY_SQLPARA_SUM = "query-sqlPara-sum";
    protected static String KEY_PARAMS = "query-params";

    protected SwMap params;
    protected UserSession us;
    private SessionCache cacheBean;
    private List<FooterField> footerFields = null;

//    private SqlPara sqlPara = null;

    @Override
    public R doWork(SwMap params, UserSession us) throws Exception {
        this.params = params;
        this.us = us;

        return R.success(listData());
    }

    protected SwListData listData() throws Exception {
        List<SwMap> listData = new ArrayList<>();
        SqlPara sqlPara = buildDataSql();

        int rows = params.readInt("rows", 20);
        int page = params.readInt("page", 1);
        String sort = params.readString("sort");
        String order = params.readString("order");
        String sql;
        if (StringUtils.isEmpty(sort)) sql = sqlPara.sql;
        else {
            sql = "select ar.* from (" + sqlPara.sql + ") ar order by ar." + sort + " " + order;
            if (StringUtils.isNotEmpty(getPkFieldName())) {
                sql += "," + getPkFieldName();
            }
        }

        if (rows == 0) {
            listData = DbEngine.getInstance().query(sql, SwMap.class, sqlPara.paras);
        } else {
            //查分页数据
            listData = DbEngine.getInstance().pagedQuery(sql, SwMap.class, (page - 1) * rows, rows, sqlPara.paras);
        }

        afterQuery(listData);
        return SwListData.create(listData, false);
    }

    protected SqlPara buildSumSqlPara() {
        final SessionCache userCache = getCacheBean();
        SqlPara sqlPara = userCache.get(KEY_SQLPARA);
        if (sqlPara == null) return null;

        getFooterFields();
        StringBuilder sql = new StringBuilder(128);
        sql.append("select count(1) F_1");
        int n = 2;
        for (FooterField cn : footerFields) {
            if (!FooterField.STATIC_TEXT.equals(cn.type)) {
                if (FooterField.SUM_TYPE_COUNT_ALL.equals(cn.type)) {
                    sql.append(",COUNT(");
                } else if (FooterField.SUM_TYPE_SUM_NUM.equals(cn.type)) {
                    sql.append(",SUM(");
                } else {
                    sql.append(",").append(cn.type).append("(");
                }
                if (FooterField.SUM_TYPE_COUNT.equals(cn.type))
                    sql.append("distinct ");
                sql.append(cn.field).append(") F_").append(n++);
            }
        }
        sql.append(" from (").append(sqlPara.getSqlNoOrderBy()).append(") xxxxxz");

        return new SqlPara(sql.toString(), sqlPara.paras);
    }

    public R getTotal(SwMap params, UserSession us) {
        try {
            this.params = params;
            this.us = us;

            final SessionCache userCache = getCacheBean();
            SqlPara sqlParaSum = userCache.get(KEY_SQLPARA_SUM);
            if (sqlParaSum == null) {
                Map<String, Object> map = userCache.get(KEY_PARAMS);
                if (map != null) params.putAll(map);
                sqlParaSum = buildSumSqlPara();
            }
            if (sqlParaSum == null) return R.success();
            userCache.put(KEY_SQLPARA_SUM, sqlParaSum);


            Map<String, Object> mapFooter = new HashMap<>();
            getFooterFields();

            int total = DbEngine.getInstance().query(sqlParaSum.sql, new ResultSetExtractor<Integer>() {
                public Integer extractData(ResultSet rs) throws SQLException {
                    try {
                        for (FooterField c : footerFields) {
                            if (FooterField.STATIC_TEXT.equals(c.type))
                                mapFooter.put(c.field, c.text);
                        }
                        int count = 0;

                        if (rs.next()) {
                            count = rs.getInt(1);
                            int n = 2;
                            for (FooterField c : footerFields) {
                                switch (c.type) {
                                    case FooterField.STATIC_TEXT:
                                        mapFooter.put(c.field, c.text);
                                        break;
                                    case FooterField.SUM_TYPE_COUNT:
                                    case FooterField.SUM_TYPE_COUNT_ALL:
                                        mapFooter.put(c.field, String.valueOf(rs.getInt(n++)));
                                        break;
                                    case FooterField.SUM_TYPE_SUM_NUM:
                                        mapFooter.put(c.field, String.valueOf(rs.getInt(n++)));
                                        break;
                                    default:
                                        mapFooter.put(c.field, rs.getDouble(n++));
                                        break;
                                }
                            }
                        } else {
                            for (FooterField c : footerFields) {
                                if (FooterField.STATIC_TEXT.equals(c.type))
                                    mapFooter.put(c.field, c.text);
                            }
                        }
                        return count;
                    } catch (Exception e) {
                        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        return 0;
                    }
                }
            }, sqlParaSum.paras);

            R r = R.success();
            r.put("total", total);
            r.put("footer", mapFooter);
            return r;
        } catch (Exception e) {
            return R.error("计算合计失败！", e);
        }
    }


    public SessionCache getCacheBean() {
        if (cacheBean == null)
            cacheBean = SessionCacheFactory.getInstance().getUserCache(us.getUserId());
        return cacheBean;
    }

    public void clearCache() {
        getCacheBean().clear();
    }

    protected SqlPara buildDataSql() throws Exception {
        SessionCache userCache = getCacheBean();

        //从缓存里看看有没有保存的sql缓存；
        SqlPara sqlParaInCache = getCacheBean().get(KEY_SQLPARA);  //缓存里面的对象；
        boolean query = sqlParaInCache == null || params.readBool("query");
        if (!query) {
            Map<String, Object> map = userCache.get(KEY_PARAMS);
            if (map == null) query = true;
            else {
                Map<String, Object> params = getCachedParams();
                if (params.size() > map.size()) query = true;//本次参数更多
                else {
                    for (String key : params.keySet()) {
                        Object v = map.get(key);
                        String cv = v != null && StringUtils.isNotEmpty(v.toString()) ? v.toString() : "";
                        v = params.get(key);
                        String rv = v != null && StringUtils.isNotEmpty(v.toString()) ? v.toString() : "";
                        if (!cv.equals(rv)) {
                            query = true;
                            break;
                        }
                    }
                }
            }
        }
        SqlPara sqlPara = null;
        if (query) {//新查询
            //缓存里面没有对象，重新构建查询条件，重新计算汇总数；
            buildParam();//构造过滤条件，设默认值
            sqlPara = buildSqlPara();  //重新构建查询对象；
            userCache.remove(KEY_SQLPARA_SUM);

            userCache.put(KEY_SQLPARA, sqlPara);
            userCache.put(KEY_PARAMS, getCachedParams());//不缓存页码之类的东东，避免下次请求时，页码又回去了
        } else {
            sqlPara = sqlParaInCache;

            Map<String, Object> map = userCache.get(KEY_PARAMS);
            if (map != null) params.putAll(map);
        }
        return sqlPara;
    }

    /**
     * 构造过滤条件，设默认值
     *
     * @throws Exception
     */
    protected void buildParam() throws Exception {

    }

    /**
     * sql主键字段，用于点击表头排序时保证排序记录唯一性
     *
     * @return
     */
    protected String getPkFieldName() {
        return null;
    }

    protected void afterQuery(List<SwMap> listData) {
    }

    protected abstract SqlPara buildSqlPara() throws Exception;

    //构建合计字段
    private List<FooterField> getFooterFields() {
        if (footerFields == null) {
            footerFields = new ArrayList<>();
            buildFooterFields(footerFields);
        }
        return footerFields;
    }

    protected void buildFooterFields(List<FooterField> footer) {
    }

    protected Map<String, Object> getCachedParams() {
        Map<String, Object> map = new HashMap<>();
        map.putAll(params);
        map.remove("page");
        map.remove("rows");
        map.remove("sort");
        map.remove("order");
        map.remove("tokenkey");
        map.remove("tokenval");
        map.remove("_v");
        map.remove("_dn");
        map.remove("_sk");
        map.remove("_skw");
        map.remove("_tk");
        map.remove("_sb");
        map.remove("__exp_model");
        map.remove("model");
        map.remove("data");
        return map;
    }
}
