package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.SwException;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelIndex;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

/**
 * Created by Akmm at 2022/3/2 19:52
 * 默认实体实现
 */
public class DefaultSaveHandler<T extends DefaultEntity> extends AbstractSaveHandler<T> {
    protected String tableName;

    public DefaultSaveHandler(String tableName) {
        this.tableName = tableName;
    }

    @Override
    protected T createComp() throws Exception {
        final EntityDao dao = DbEngine.getInstance().findDao(tableName);
        T bean = (T) dao.createBean();
        bean.setEntityId(dao.nextId());
        return bean;
    }

    @Override
    protected T loadComp(long id) {
        return new DefaultProvider<T>(tableName).getBean(id);
    }

    @Override
    protected void readFromPage() {
        bean.readFromMap(params);
    }

    @Override
    protected void checkValid() throws Exception {
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        for (ModelField field: table.getFields()) {
            if (field.isNotNull() && StringUtils.isEmpty(bean.getStr(field.getName()))) {
                throw new SwException(field.getTitle() + "不能为空！");
            }
        }

        EntityDao dao = DbEngine.getInstance().findDao(tableName);
        for (ModelIndex mi: table.getIndexes()) {
            if (mi.isUnique()) {
                dao.checkUnique(bean, mi.getFields().split(","));
            }
        }
    }

    @Override
    protected void save() {
        EntityDao dao = DbEngine.getInstance().findDao(tableName);
        if (isNew) {
            dao.insertEntity(bean);
        } else {
            dao.updateEntity(bean);
        }
    }

    @Override
    protected void saveSuccess() {
        super.saveSuccess();
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table.isNeedCache()) {
            AbstractCache<T> cache = CacheManager.getIntance().getCache(tableName);
            cache.put(bean);
        }
    }

    @Override
    protected void saveFailed() {
        super.saveFailed();
        ModelTable table = ModelTableCache.getInstance().getByName(tableName);
        if (table.isNeedCache()) {
            AbstractCache<T> cache = CacheManager.getIntance().getCache(tableName);
            cache.reset(bean);
        }
    }
}
